require_dependency "skirental_payment/application_controller"
require 'paypal-sdk-rest'
include PayPal::SDK::REST
include PayPal::SDK::Core::Logging

module SkirentalPayment
    class PaypalPaymentsController < PaymentsController

        def create
            booking = SkirentalFrontendApi::Booking.where(shop_id: paypal_payment_params[:shop_id], session_id: paypal_payment_params[:session_id]).first
            fail if booking.nil?

            config = Rails.application.config.payments[:PAYPAL]

            transaction = booking.transaction

            if config[:mode] == "sandbox"
                paypal_ids = config
            else
                #TODO: FILL FROM API!
                fail "GET FROM SR-API!!"
            end

            PayPal::SDK::REST.set_config(
                :mode           => config[:mode], # "sandbox" or "live"
                :client_id      => paypal_ids[:client_id],
                :client_secret  => paypal_ids[:client_secret]
            )

            payment_context = SkirentalPayment::Payment.new(booking_id: booking.id, is_payed: false, transaction: transaction)

            # Build Payment object
            @payment = PayPal::SDK::REST::Payment.new({
                                                        :intent =>  "sale",
                                                        :payer =>  {
                                                            :payment_method =>  "paypal"
                                                        },
                                                        :redirect_urls => {
                                                            :return_url => "#{request.protocol}#{request.host_with_port}",
                                                            :cancel_url => "#{request.protocol}#{request.host_with_port}"
                                                        },
                                                        :transactions => [transaction]
            })

            if @payment.create
                payment_context.payment_reference = @payment.id # Payment Id
                if payment_context.save
                    render json: { paymentID: @payment.id  }, status: :created
                else
                    render json: { error: payment_context.errors.full_messages }, status: :ok
                end
            else
                @payment.error  # Error Hash
                render json: { error: @payment.error  }, status: :bad_request
            end

        end

        # Is called by Client (Paypal-Checkout-Flow Window) after payer presses "Pay Now"
        def execute
            @payment = PayPal::SDK::REST::Payment.find(paypal_payment_params[:payment_id])

            # PayerID is required to approve the payment.
            if @payment.execute( :payer_id => paypal_payment_params[:payer_id] )
                logger.info "Payment[#{@payment.id}] execute successfully"

                payment_context = SkirentalPayment::Payment.where(payment_reference: @payment.id).first
                fail if payment_context.nil?

                payment_context.is_payed = true;
                if payment_context.save
                    confirm_payment(payment_context)
                    render json: { }, status: :ok
                else
                    render json: { error: payment_context.errors.full_messages }, status: :ok
                end
            else
                logger.error @payment.error.inspect
                render json: { error: @payment.error.inspect }, status: :bad_request
            end
        end

        private

        def paypal_payment_params
            params.permit(:shop_id, :session_id, :payment_id, :payer_id)
        end
    end
end
